package analyseplans;
import static org.nuiton.i18n.I18n._;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import scripts.ResultName;
import java.io.*;
import java.io.File;
import java.io.FileReader;
import java.io.Writer;
import java.util.*;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import org.nuiton.math.matrix.*;
import org.nuiton.util.*;
import org.nuiton.topia.*;
import fr.ifremer.isisfish.*;
import fr.ifremer.isisfish.types.*;
import fr.ifremer.isisfish.rule.Rule;
import fr.ifremer.isisfish.rule.RuleHelper;
import fr.ifremer.isisfish.simulator.SimulationContext;
import fr.ifremer.isisfish.types.Date;
import fr.ifremer.isisfish.entities.*;
import fr.ifremer.isisfish.simulator.AnalysePlan;
import fr.ifremer.isisfish.simulator.AnalysePlanContext;
import fr.ifremer.isisfish.simulator.AnalysePlanIndependent;
import fr.ifremer.isisfish.simulator.SimulationParameter;
import fr.ifremer.isisfish.datastore.RegionStorage;
import fr.ifremer.isisfish.datastore.RuleStorage;
import fr.ifremer.isisfish.datastore.SimulationStorage;
import fr.ifremer.isisfish.datastore.ResultStorage;

/*
 * SAAnchois2tac.java
 *
 * Created: 6 juin 2007
 *
 * @author bpoussin <bpoussin@labs.libre-entreprise.org>
 * @version $Revision: 1.27 $
 *
 * Last update: $Date: 2007/11/02 $
 *ajout des instructions pour modifier la regle Paita
 * by : $Author:sigrid $
 */


public class HindcastUNCOVER implements AnalysePlanIndependent {

    /** to use log facility, just put in your code: log.info("..."); */
    static private Log log = LogFactory.getLog(HindcastUNCOVER.class);
   
    static private final String MATRIX = "matrixHc";    
    static private final String TACe = "tacEsp";
    static private final String TACMPAf = "TacMpaRum";
    static private final String AMPe = "mpaEsp";
    static private final String RedEff = "redEff";
	static private final String PAITA = "paita";
    
    int parameterNumber = 5;
    public int param_first = 0;
    public int param_simulationNumber = 16;
    public String param_directory = "Inputs_Anchois/ScenariosUNCOVERHindcast/";
    private MatrixND matrix = null;

    public String [] necessaryResult = {
    // put here all necessary result for this rule
    // example: 
    // ResultName.MATRIX_BIOMASS,
    // ResultName.MATRIX_NET_VALUE_OF_LANDINGS_PER_STRATEGY_MET,
    };

    public String[] getNecessaryResult() {
        return this.necessaryResult;
    }

    /**
     * Permet d'afficher a l'utilisateur une aide sur le plan.
     * @return L'aide ou la description du plan
     */
    public String getDescription() throws Exception {
        return _("Hindcast scenarios for UNCOVER");
    }
 
 
    public void init(AnalysePlanContext context) throws Exception {
        File dir = new File(param_directory);
        
        matrix = MatrixFactory.getInstance().create(new int[]{param_simulationNumber, parameterNumber});
        matrix.importCSV(new FileReader(new File(dir, MATRIX + ".txt")), new int[]{0,0});
        matrix.setSemantic(1, Arrays.asList(new String[]{ TACMPAf,AMPe,TACe,RedEff,PAITA}));
              
        context.getParam().addExtraRules("Paita5_Anchois","Cantonnement","FrTACfixMPARumMultiCost_Anchois","FrRuleBMPARumMultiCost_Anchois","TACfix_Es_Anchois","RuleB_Es_Anchois","EffortReduction","TotalClosure");
    }
     
    /**
     * @param name le nom de l'element a recuperer
     * @param simulation le numero de la simulation
     * @return
     */
    private double getDouble(String name, int ligne) throws Exception {
        File dir = new File(param_directory);
        Properties prop = new Properties();
        prop.load(new BufferedReader(new FileReader(new File(dir, name + ".txt"))));
		int mod = (int)matrix.getValue(ligne, name);
        double result = Double.parseDouble(prop.getProperty(""+mod));
        return result;
    }
	
    /**
     * @param name le nom de l'element a recuperer
     * @param simulation le numero de la simulation
     * @return
     */    
    private double [] getList(String name, int ligne) throws Exception {
        File dir = new File(param_directory);
        Properties prop = new Properties();
        prop.load(new BufferedReader(new FileReader(new File(dir, name + ".txt"))));
        int mod = (int)matrix.getValue(ligne, name);
        double [] result = StringUtil.toArrayDouble(prop.getProperty(""+mod).split(";"));
        return result;
    }
 
    /**
     * @param name le nom de l'element a recuperer
     * @param simulation le numero de la simulation
     * @return
     */
    private String getString(String name, int ligne) throws Exception {
        File dir = new File(param_directory);
        Properties prop = new Properties();
        prop.load(new BufferedReader(new FileReader(new File(dir, name + ".txt"))));
        int mod = (int)matrix.getValue(ligne , name);
        String result = prop.getProperty(""+mod);
        return result;
    }
  
    /**
     * @param name le nom de l'element a recuperer
     * @param simulation le numero de la simulation
     * @return
     */  
    private Rule getRule(TopiaContext tc, String name, int ligne) throws Exception {
        File dir = new File(param_directory);
        Properties prop = new Properties();
        prop.load(new BufferedReader(new FileReader(new File(dir, name+".txt"))));
        int mod = (int)matrix.getValue(ligne , name);
        String ruleName = prop.getProperty(""+mod);
		System.out.println("ruleName : "+ ruleName);
        RuleStorage ruleStorage = RuleStorage.getRule(ruleName);
        Rule rule = ruleStorage.getNewRuleInstance();
        RuleHelper.populateRule(mod, tc, rule, prop);
        return rule;
    }
    
    /**
     * Call before each simulation
     * @param context plan context
     * @param nextSimulation storage used for next simulation
     * @return true if we must do next simulation, false to stop plan
     * @throws Exception
     */
    public boolean beforeSimulation(AnalysePlanContext context, SimulationStorage nextSimulation) throws Exception {
        int simNum = nextSimulation.getParameter().getAnalysePlanNumber(); 
		int ligne = simNum + param_first ;
        if (ligne < param_simulationNumber) {
        
	        Rule tacE = getRule(nextSimulation.getStorage(), TACe, ligne);
	        Rule tacMpaRumF = getRule(nextSimulation.getStorage(), TACMPAf, ligne);
			Rule ampE = getRule(nextSimulation.getStorage(), AMPe, ligne);
	        Rule redEffort = getRule(nextSimulation.getStorage(),RedEff, ligne);
	        Rule Paita = getRule(nextSimulation.getStorage(), PAITA, ligne);
			// modif les parametres des regles de gestion
	        List<Rule> paramRules = nextSimulation.getParameter().getRules();

            // on ajoute les regles
            paramRules.add(tacE);
            paramRules.add(ampE);
		  paramRules.add(tacMpaRumF);
            paramRules.add(redEffort);
            paramRules.add(Paita);

            // modif la base de donnÃƒÆ’Ã†â€™Ãƒâ€šÃ‚Â©es
            return true;
        } else {
            return false;
        }
    }

    /**
     * Call after each simulation
     * @param context plan context
     * @param nextSimulation storage used for next simulation
     * @return true if we must do next simulation, false to stop plan
     * @throws Exception
     */
    public boolean afterSimulation(AnalysePlanContext context, SimulationStorage lastSimulation) throws Exception {
        return true;
    }

}
